/*------------------------------------------------------------------*/
/* File: FrExpand.c   v3.74                            D. Verkindt  */
/*                                        Last update: 10 Feb. 1999 */
/*                                                                  */  
/*  This program gets frames from one single file and put them      */
/*  as one frame per file RUN#_FR# inside the output directory      */
/*  Syntax is: FrExpand  -i <input file>                            */
/*                       -o <output directory>                      */
/*                       -r <begin run number> <end run number>     */
/*                       -f <begin frame number> <end frame number> */
/*                       -R (if you want one Run per File           */
/*                           instead of one Frame per File)         */
/*                       -debug <debug level>                       */
/*                       -h (to get help)                           */
/*------------------------------------------------------------------*/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>

#include "FrameL.h"

int debug = 0;

#define LBUFF_IN     540000
#define LSTRING_MAX  256
#define DEFAULT1    -1
#define DEFAULT2     10000000

struct Parameters{
  char *inName;     /* Name of the input file       */
  char *outName;    /* Name of the output file      */
  long run1;        /* copy from run1 to run2       */
  long run2;        /* copy from run1 to run2       */
  long frame1;      /* copy from frame1 to frame2   */
  long frame2;      /* copy from frame1 to frame2   */
  long byrun;       /* output = one run per file    */
};

void warning(int arg);
void SetParameters(int argc, char **argv, struct Parameters *par);
char *my_strcpy(char *dummy, char *str);

struct Parameters *par;




/*-------------------------------------------------------- main ------------*/

int main(int argc, char **argv)
{long  previousRun, cpt, runcpt;
 int result;
 char  *buffI,*buffO=NULL;
 struct FrFile *oFile=NULL;
 struct FrFile *iFile;
 struct FrameH *frame;
 char  outfileName[300];


 /*----- Deal with input arguments -----*/
 par = (struct Parameters *)calloc(1,sizeof(struct Parameters));
 SetParameters(argc, argv, par);

 if (debug>1)
    {printf("\n Arguments used:\n"); 
     printf("   inName        = %s \n",           par->inName);
     printf("   outName       = %s \n",           par->outName);
     printf("   runNumbers    = %ld  to  %ld \n", par->run1, par->run2);
     printf("   frameNumbers  = %ld  to  %ld \n", par->frame1, par->frame2);
     printf("   oneRunperFile = %ld \n",           par->byrun);
     sleep(1);
    }

 
 /*----- create output directory -----*/

 result = mkdir(par->outName,0744);

 if (result != 0)
    {printf("Could not create the directory %s. I stop.\n",par->outName);
     exit(0);
    }

 printf("\n\n");
 printf("--------------------------------------------------------------------------\n");
 printf("   Expanding frames %ld to %ld \n", par->frame1, par->frame2);
 printf("   from file      %s \n", par->inName);
 printf("   to   directory %s \n", par->outName);
 printf("--------------------------------------------------------------------------\n");


 /*----- open the input file -----*/

 buffI = malloc(LBUFF_IN);
 iFile = FrFileINew (par->inName, buffI, LBUFF_IN);
 if(iFile == NULL)
    {printf("Error during file opening\n"
            "  Last errors are:\n%s",FrErrorGetHistory());
     warning(0);
    }



 /*--- Main Loop: read frames and write each of them as a file inside the output directory ---*/

 previousRun = -1;
 runcpt = 0;
 cpt    = 0;

 while((frame = FrameRead(iFile)) != NULL)
   {if ((frame->run == par->run2 || par->run2 == DEFAULT2) && frame->frame > par->frame2)
            {FrameFree(frame);
             break;
            }
    if (frame->run < par->run1 || frame->run > par->run2 ||
        frame->frame < par->frame1 || frame->frame > par->frame2)
            {FrameFree(frame);
             continue;
            }

    if (par->byrun && previousRun != frame->run)
         {previousRun = frame->run;
          runcpt++;
          sprintf(outfileName,"%s/RUN%.3d",par->outName,frame->run);
          if (oFile) FrFileOEnd(oFile);
          if (buffO) free(buffO);
          buffO     = malloc(LBUFF_IN);
          oFile = FrFileONew (outfileName, 3, buffO, LBUFF_IN);
          if(oFile == NULL)
             {printf("Error during output file creation\n"
                     "  Last errors are:\n%s",FrErrorGetHistory()); warning(0);}
          FrFileOSetGzipLevel(oFile, 1);
         }

    if (!par->byrun)
         {sprintf(outfileName,"%s/RUN%.3d_FR%.5d",par->outName,frame->run,frame->frame);
          buffO     = malloc(LBUFF_IN);
          oFile = FrFileONew (outfileName, 3, buffO, LBUFF_IN);
          if(oFile == NULL)
              {printf("Error during output file creation\n"
                      "  Last errors are:\n%s",FrErrorGetHistory());
               warning(0);
              }
          FrFileOSetGzipLevel(oFile, 1);
         }

 
      if(FrameWrite(frame, oFile) != FR_OK)
          {printf("Error during frame write\n"
                   "  Last errors are:\n%s",FrErrorGetHistory());
           exit(0);
          }
      cpt++;

      if (debug>1) 
        {printf("    Run %d Frame %d put into file %s\n",
                frame->frame, frame->run, outfileName);}

      FrameFree(frame);

      if (!par->byrun)
         {FrFileOEnd(oFile);
          free(buffO);
         }
    }


 /*----- termination -----*/

 FrFileIEnd(iFile);
 free(buffI);
 if (cpt == 0)
    {printf("\n\n Nothing done. Run or Frame numbers are out of range.\n");}
 else if (par->byrun ==0)
    {printf("\n\n %ld frames created in directory %s \n",cpt,par->outName);}
 else if (par->byrun > 0)
    {printf("\n\n %ld runs (%ld frames) created in directory %s \n",runcpt,cpt,par->outName);}
 printf("\n -- OK. FrExpand Done.\n\n");
 exit(0);
}





/*-------------------------------------------------------------- warning ------*/

void warning(int arg)
{
 if (arg==1)
   {printf("\n---------------------------------------------------------------------\n");
    printf("* Warning: file has not been read. May be, it does not contain frames! *\n");
    printf("-----------------------------------------------------------------------\n\n");
   }
 if (arg==0)
   {printf("\n\n--------------------------------------------------------------------------\n");
    printf("          FrExpand             \n");   
    printf("      A problem occured        \n");   
    printf("                               \n");
    printf("      ! W A R N I N G !        \n");
    printf("                               \n");
    printf("      type FrExpand -h         \n"); 
    printf("        to get help            \n");
    printf("                               \n"); 
    printf("--------------------------------------------------------------------------\n\n");
    exit(0);
   }
 return;
}





/*-------------------------------------------------------------- SetParameters ------*/

void SetParameters(int argc, char **argv, struct Parameters *par)
{int i;

 if (debug>10) printf("\n SetParameters \n");

 /*-------- default values ---------*/
 par->inName  = NULL;
 par->outName = NULL;
 par->run1    = DEFAULT1;
 par->frame1  = DEFAULT1;
 par->run2    = DEFAULT2;
 par->frame2  = DEFAULT2;
 par->byrun   = 0;


 /*-------- online help --------*/
  if (argc==1 || (argc>1 && strncmp(argv[1],"-h",2) == 0))
    {printf(" \n");
     printf("------------------------------------------------------------------------ \n");
     printf(" This program gets frames from one single file and put them as one       \n");
     printf(" frame per file or one run per file in the output directory.             \n");
     printf("------------------------------------------------------------------------ \n");
     printf(" Syntax is:   FrExpand  -i     <input file>                              \n");
     printf("                        -o     <output directory>                        \n"); 
     printf("                        -r     <begin run number> <end run number>       \n"); 
     printf("                        -f     <begin frame number> <end frame number>   \n");
     printf("                        -byrun <0 or 1>        (1 = one run per file)    \n");
     printf("                        -debug <debug level>                             \n");
     printf("                        -h   (to get help)                               \n");
     printf("------------------------------------------------------------------------ \n");
     printf("    -i     : Input file containing the frames to expand.                 \n");
     printf("    -o     : Output directory.                                           \n");
     printf("             Located by default at the same level as the input file.     \n");
     printf("    -r     : Giving one run number is like giving twice the same         \n");
     printf("    -f     : Giving one frame number is like giving twice the same       \n");
     printf("    -byrun : If you want one run per file instead of                     \n");
     printf("             one frame per file (default = 0 = one frame per file)       \n");
     printf("------------------------------------------------------------------------ \n");
     printf(" \n");
     exit(0);
    }

 if (argc == 2)
       {par->inName = my_strcpy(par->inName, argv[1]); argc = 1;}

 for (i=1; i<argc-1; i++)
    {if (strcmp(argv[i],"-i") == 0)
         {par->inName = my_strcpy(par->inName, argv[i+1]);
          i++;
         }
     if (strcmp(argv[i],"-o") == 0)
         {par->outName = my_strcpy(par->outName, argv[i+1]);
          i++;
         }
     if (strcmp(argv[i],"-r") == 0)
         {par->run1   = atoi(argv[i+1]);
          if (i+2 == argc)
             {par->run2 = par->run1;
              i++;
              break;
             }
          if (atoi(argv[i+2]) != 0)
             {par->run2 = atoi(argv[i+2]);
              i=i+2;
             }
          else
             {par->run2 = par->run1;
              i=i+1;
             }
         }
     if (strcmp(argv[i],"-f") == 0)
         {par->frame1 = atoi(argv[i+1]);
          if (i+2 == argc)
             {par->frame2 = par->frame1;
              i++;
              break;
             }
          if (atoi(argv[i+2]) != 0)
             {par->frame2 = atoi(argv[i+2]);
              i=i+2;
             }
          else
             {par->frame2 = par->frame1;
              i++;
             }
         }
     if (strcmp(argv[i],"-byrun") == 0)
         {par->byrun = atoi(argv[i+1]);
          i++;
         }
     if (strcmp(argv[i],"-debug") == 0)
         {debug= atoi(argv[i+1]);
          i++;
         }
    }


 if (!par->inName)
    {printf("\n  You must give me at least a file name (option -i)\n\n");
     exit(0);
    }

 if (!par->outName)
    {par->outName = (char*)calloc(strlen(par->inName)+5,sizeof(char));
     sprintf(par->outName,"%s_dir",par->inName);
    }

 return;
}




/*---------------------------------------------------- my_strcpy ---------*/
char *my_strcpy(char *dummy, char *str)
{
 if (debug>200) printf("\nmy_strcpy \n");

 if (!str) return(NULL);

 if (dummy) free(dummy);
 dummy = (char *) calloc(strlen(str)+1,sizeof(char));
 strcpy(dummy, str);

 return(dummy);
}

