/*--------------------------------------------------------------------*/
/* This file contains all the I/O low level function used by the Frame*/
/* Library.To use special IO devices you just need to change this file*/
/*     (Code proposed by Roy Willams, Caltech) (Jun 14, 1999 version) */
/*--------------------------------------------------------------------*/
#include <stdlib.h>
#include <stdio.h>
#include "FrIO.h"

/*------------------------------------ this is need for Windows NT --*/
#if !defined(O_BINARY)
#define O_BINARY 0
#endif
/*--------------------------------------------------------------------*/
struct FrIO *FrIOOpenR(char* filename)
/*--------------------------------------------------------------------*/
{struct FrIO *frfd;
	
 frfd = (struct FrIO *)malloc(sizeof(struct FrIO));
 if(frfd == NULL) return(NULL);

#ifdef FRIOCFILE
 frfd->fp = fopen(filename, "r");
 if(frfd->fp == NULL) {return (NULL);}
#else
 frfd->fd = open(filename, O_RDONLY|O_BINARY, 0);
 if(frfd->fd < 0) {return (NULL);}
#endif

 else                 {return (frfd);}
}

/*--------------------------------------------------------------------*/
struct FrIO *FrIOOpenW(char* filename)
/*--------------------------------------------------------------------*/
{struct FrIO *frfd;
	
 frfd = (struct FrIO *)malloc(sizeof(struct FrIO));
 if(frfd == NULL) return(NULL);

#ifdef FRIOCFILE
 frfd->fp = fopen(filename, "w");
 if(frfd->fp == NULL) {return (NULL);}
#else
 unlink(filename);
 frfd->fd = open(filename,  O_CREAT|O_WRONLY|O_BINARY,0644);
 if(frfd->fd < 0) {return (NULL);}
#endif

 else             {return (frfd);}
}
/*--------------------------------------------------------------------*/
int FrIOClose(struct FrIO *frfd)
/*--------------------------------------------------------------------*/
{int ret;

#ifdef FRIOCFILE
 ret = fclose(frfd->fp);
#else
 ret = close(frfd->fd);
#endif

 free(frfd);
 return ret;
}

/*--------------------------------------------------------------------*/
int FrIOWrite(struct FrIO *frfd, char* buf, int nbyte)
/*--------------------------------------------------------------------*/
{
#ifdef FRIOCFILE
 return fwrite(buf, 1, nbyte, frfd->fp);
#else
 return write(frfd->fd, buf, nbyte);
#endif
}

/*--------------------------------------------------------------------*/
int FrIORead(struct FrIO *frfd, char* buf, int nbyte)
/*--------------------------------------------------------------------*/
{
#ifdef FRIOCFILE
 return(fread(buf, 1, nbyte, frfd->fp));
#else
 read(frfd->fd, buf, nbyte);

 return(nbyte);
#endif
}

/*--------------------------------------------------------------------*/
int FrIOSet(struct FrIO *frfd, long offset)
/*--------------------------------------------------------------------*/
/* Set the file position (in bytes) from the file beggining
   returns -1 in case of error       */
{long off;

#ifdef FRIOCFILE
 off = fseek(frfd->fp, offset, SEEK_SET);
#else
 off = lseek(frfd->fd, offset, SEEK_SET);
#endif

 return(off);}

/*--------------------------------------------------------------------*/
int FrIOSetFromEnd(struct FrIO *frfd, long offset)
/*--------------------------------------------------------------------*/
/* Set the file position (in bytes) from the end of file
   returns -1 in case of error       */
{long off;

#ifdef FRIOCFILE
 off = fseek(frfd->fp, offset, SEEK_END);
#else
 off = lseek(frfd->fd, offset, SEEK_END);
#endif

 return(off);}

/*--------------------------------------------------------------------*/
int FrIOSetFromCur(struct FrIO *frfd, long offset)
/*--------------------------------------------------------------------*/
/* Set the file position (in bytes) from the end of file
   returns -1 in case of error       */
{long off;

#ifdef FRIOCFILE
 off = fseek(frfd->fp, offset, SEEK_CUR);
#else
 off = lseek(frfd->fd, offset, SEEK_CUR);
#endif

 return(off);}






