/* *******************************************************************
********** FrMark.h **************************** FrMark.h ************
********** FrMark.h **************************** FrMark.h ************
******************************************************************* */
/*  by B.Mours LAPP (Annecy); June 9, 1999*/

#ifdef __cplusplus
extern "C" {
#endif

               
#define FRMAPMAXSH 100

struct FrFileMark;
struct FrStatMark;
struct FrDataMark;
struct FrFile;
struct FrameH;

struct FrFileMark {       /* fast access information                    */
  struct FrSH *classe;    /* class information (internal structure)     */
  unsigned short ULeapS;  /* leap seconds (GPS vs UTC) (first frame)    */
  int localTime;          /* Time offset = Local time-UTC(sec)(f.frame) */
  unsigned int nFrame;    /* number of frames stored                    */
  unsigned int *run;      /* list of run numbers                        */
  unsigned int *frame;    /* list of frame numbers                      */
  double *GTime;          /* list of GPS starting time (second)         */
  long *frPos;            /* list of frame position from the start file */
  int nSH;                /* number of structure header                 */
  unsigned short *SHid;   /* list of SH id's                            */
  char **SHname;          /*list of structures names                    */
  long position;          /* offset from the end of file                */
  int nStat;              /* number of static Data blocks               */
  struct FrStatMark *statMark;  /* file marks for Static Data           */
  int nAdc;               /* number of ADC blocks                       */
  struct FrDataMark *adcMark;/* file marks for AdcData                  */
  struct FrDataMark *proMark;/* file marks for ProData                  */
  struct FrDataMark *simMark;/* file marks for SimData                  */
  struct FrDataMark *triMark;/* file marks for SimData                  */
                          /* ------ end_of_SIO parameters --------------*/
  int maxFrame;           /* size of the following vectors              */
};

struct FrDataMark {      /* hold Simulated data                         */
  char   *name;          /* name                                        */
  long   *position;      /* indexed for each frame                      */
  struct FrDataMark *next;/* next structure                              */
                         /* ------ end of SIO parameters ---------------*/
    }; 

struct FrStatMark { /*-- direct access data for FrStatData   --------------*/
  char *name;               /* static data bloc name                       */
  char *detector;           /* detector name                               */
  int nStat;                /* number of instance                          */
  unsigned int *tStart;     /* array (nStat) of starting time (GPS time)   */
  unsigned int *tEnd;       /* array (nStat) of validity end time(GPS time)*/
  unsigned int *version;    /* array (nStat) of version number             */
  long *position;           /* array (nStat) of position in the file       */
			    /* ------ end_of_SIO parameters ---------------*/
  int maxStat;              /* maximum number of instance                  */
  struct FrStatMark *next;  /* next static data bloc                       */
 };

/*----------------------------------------------------------------------*/
struct FrAdcData *FrAdcDataReadN(struct FrFile *iFile,char *name,
                                 int runN, int frameN);
struct FrAdcData *FrAdcDataReadT(struct FrFile *iFile,char *name,
                                 double gtime, double dt);
struct FrameH *FrameReadN(struct FrFile *iFile,int runN,int frameN);
struct FrameH *FrameReadT(struct FrFile *iFile,double gtime, double dt);
/*----------------------------------------------------------------------*/
struct FrSH *FrFileMarkDef();
void FrFileMarkGetStatD(struct FrFile *iFile, struct FrameH *frame);
void FrFileMarkDump(struct FrFileMark *mark);
void FrFileMarkFree(struct FrFileMark *mark);
void FrFileMarkData(struct FrFile *oFile,
                    struct FrDataMark *firstMark, char *name);
void FrFileMarkSH(struct FrFile *oFile, char *name, unsigned short id);
void FrFileMarkFrame(struct FrFile *oFile, struct FrameH *frame);
void FrFileMarkNew(struct FrFile *oFile, struct FrameH *frame);
struct FrFileMark* FrFileMarkRead(struct FrFile *iFile);
void FrFileMarkWrite(struct FrFile *oFile);
/*----------------------------------------------------------------------*/

#ifdef __cplusplus
}
#endif
















