/* *******************************************************************
********** FrameL.h **************************** FrameL.h ************
********** FrameL.h **************************** FrameL.h ************
******************************************************************* */
/*  by B.Mours LAPP (Annecy); September 11, 1999*/
#define FRAMELIB_VERSION  3.85
#if !defined(FRAMELIB_DEFINED)
#define FRAMELIB_DEFINED

#ifdef __cplusplus
extern "C" {
#endif

#include <time.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "FrIO.h"
#include "FrMark.h"

typedef enum  {FR_NO, FR_YES} FRBOOL;  /*this set FR_NO=0, FR_YES=1 */

enum FrError {FR_OK,
              FR_ERROR_NO_FRAME,
              FR_ERROR_NO_FILE,
              FR_ERROR_MALLOC_FAILED,
              FR_ERROR_BUFF_OVERFLOW,
              FR_ERROR_WRITE_ERROR,
              FR_ERROR_READ_ERROR,
              FR_ERROR_FRSET,
              FR_ERROR_BAD_END_OF_FRAME,
              FR_ERROR_OPEN_ERROR};
               
#define FRMAPMAXSH 100
#define FRMAPMAXREF 40
#define FRMAPMAXINSTANCE 500
/*--------------------------- time define ------------------------------*/
#define FRGPSOFF 315964800
#define FRGPSDELTA    13
#define FRGPSTAI   19
#define FRGPSLEAPS 19+13
/* Just some rules about time .......                                   */
/* the GPS time origin is Sun Jan  6 00:00:00 1980                      */
/* the C function time origin is Jan 1 00:00:00 1970 (no leap seconds)  */
/*       (C time - GPS time = 3600*24*3567 = 315964800 seconds)         */
/*  TAI is ahead of GPS by 19 seconds                                   */
/*  TAI is ahead of UTC by 31 seconds (august 97)                       */
 /*---- GPS time = UTC - 3600*24*3657 + delta --------------------------*/
/*from 1997 01 Jul, UTC to 1999 01 January, UTC:  delta = +12s          */
/*from 1999 01 Jan, UTC until further notice:     delta = +13s          */
/*(see ftp://tycho.usno.navy.mil/time.html)                             */
/*----------------------------------------------------------------------*/

struct FrameH;
struct FrAdcData;
struct FrDetector;
struct FrEndOfFrame;
struct FrEndOfFile;
struct FrMsg;
struct FrFile;
struct FrHistory;
struct FrMap;
struct FrObject;
struct FrRawData;
struct FrProcData;
struct FrSE;
struct FrSH;
struct FrSimData;
struct FrSerData;
struct FrStatData;
struct FrVect;
struct FrTrigData;
struct FrSummary;

struct FrameH {              /* Frame header structure                 */
  struct FrSH *classe;       /* class information (internal structure) */
  char *name;                /* frame name (experiment name)           */ 
  unsigned int run;          /* run number                             */
  unsigned int frame;        /* frame number                           */
  unsigned int GTimeS;       /* frame starting Time(GPS:s.since 6/1/80)*/
  unsigned int GTimeN;       /* frame starting Time(nsec modulo 1 sec) */
  unsigned short ULeapS;     /* leap seconds between GPS and UTC       */
  int localTime;             /* Time offset = Local time - UTC (sec)   */
  double dt;                 /* frame length (nsec)                    */
  struct FrVect *type;       /* Array used to store general info like the
                                trigger type. Their meaning will be 
                                define later.                          */
  struct FrVect *user;       /*    Array used to store user data       */
  struct FrDetector *detectSim; /*detector used for simulation         */
  struct FrDetector *detectProc;/*detector used for reconstruction     */
                             /* (reconstruction = off line processing) */
  struct FrHistory *history; /* pointer to the history bank            */
  struct FrRawData *rawData; /* pointer to the raw data structure      */
  struct FrProcData *procData;/* pointer to the onstructed data        */
  struct FrProcData *strain; /* best estimate for process data         */
  struct FrSimData *simData; /* pointer to the simulated data buffers  */
  struct FrTrigData *trigData;/* pointer to the trigger data structure */
  struct FrSummary *summaryData; /* pointer to statistical summary data*/
  struct FrVect *auxData;    /* pointer to the auxiliary data          */
                             /* ------ end_of_SIO parameters ----------*/
    };

struct FrAdcData {       /* hold ADC data (or image)                    */
  struct FrSH *classe;   /* class information (internal structure)      */
  char   *name;          /* signal name                                 */
  char   *comment;       /* signal description;  the adc units should 
                            be stored in adc->data->unit[0]             */
  unsigned int crate;    /* crate number                                */
  unsigned int channel;  /* channel number                              */
  unsigned int nBits;    /* number of bits                              */
  float bias;            /* DC bias on channel.  the step 
                                  size is stored in adc->data->dx;      */  
  float slope;           /* ADC calibration: input units/count          */
  char *units;           /* ADC calibration: input units for slope      */
  double sampleRate;     /* sampling frequency (unit = Hz)              */
  unsigned int timeOffsetS;/* offset from the frame starting time (sec) */
  unsigned int timeOffsetN;/* offset from the frame starting time (nsec)*/
  double fShift;         /* frequency shift if the signal was 
                                        heterodyne (Hz) (0->no shift)   */ 
  unsigned short overRange;/* flag used to specify if the ADC input was 
                              out of range (0=ok) during this frame     */
  struct FrVect *data;   /* array of data                               */
  struct FrVect *aux;    /* array of auxiliary data if needed. Static data
                            should be stored in the FrStatData structures*/
  struct FrAdcData *next;/* next adc structure                          */
                         /* -------- end_of_SIO parameters -------------*/ 
  void *inMemory;        /* pointer to the data in the memory buffer    */
    }; 

struct FrDetector { 
  struct FrSH *classe;       /* class information (internal structure)  */
  char   *name;              /* detector name                           */ 
  short longitudeD;          /* detector longitude(est of greenwich)    */ 
  short longitudeM;          /* detector longitude(est of greenwich)    */ 
  float longitudeS;          /* detector longitude(est of greenwich)    */ 
  short latitudeD;           /* detector latitude                       */ 
  short latitudeM;           /* detector latitude                       */ 
  float latitudeS;           /* detector latitude                       */ 
  float elevation;           /* detector altitude (meter)               */ 
  float armXazimuth;         /* orient. of X arm in radian CCW from East*/ 
  float armYazimuth;         /* orient. of Y arm in radian CCW from East*/ 
  float armLength;           /* arm length (meter)                      */ 
  struct FrVect *more;       /* more data for this detector             */ 
                             /* ------ end_of_SIO parameters -----------*/ 
  struct FrStatData *sData;  /* first static data bloc                  */
 };

struct FrEndOfFrame {       /* End of Frame Marker                      */
  struct FrSH *classe;      /* class information (internal structure)   */
  unsigned int run;         /* run number                               */
  unsigned int frame;       /* frame number                             */
                            /* ------------- end_of_SIO parameters ---- */
    };

struct FrEndOfFile {        /* End of File Marker                       */
  struct FrSH *classe;      /* class information (internal structure)   */
  unsigned int nFrames;     /* number of frame                          */
  unsigned int nBytes;      /* number of bytes                          */
  unsigned int chkFlag;     /* flag for checksum                        */
  unsigned int chkSum;      /* checksum                                 */
                            /* ------------- end_of_SIO parameters ---- */
    };

struct FrMsg {              /* hold  message      information           */
  struct FrSH *classe;      /* class information (internal structure)   */
  char   *alarm;            /* name of the alarm                        */
  char   *message;          /* error message                            */
  unsigned int severity;    /* severity (to be defined)                 */
  struct FrMsg *next;       /* next structure                           */
                            /* -------- end_of_SIO parameters ----------*/ 
    }; 
    
struct FrFile{              /* standard header for i/o file             */
  char *fileList;           /* list of file name                        */
  char *fileName;           /* current file name                        */
  char *namePtr;            /* pointer in the fileList                  */
  FRBOOL string;            /* if YES output is in a string             */
  FRBOOL compress;          /* if YES compress frames at writing time   */
  unsigned short gzipLevel; /* compression level used by gzip           */ 
  struct FrSH **sh;         /* pointer to the sh struct. Used for read  */
  struct FrSH *firstSH;
  struct FrSH *curSH;
  long maxSH;               /* size of FrSHlist vector                  */
  long fmType;              /* give format conversion type              */
  long fmLong;              /* give format conversion type for long     */
  FRBOOL endOfFrame;        /* tell if we reach the end of frame ord */
  struct FrameH *curFrame;/* current frameH pointer for this file     */
  long fmtVersion;          /* format version                           */
  unsigned char header[40]; /* header for format type                   */
  long run;                 /* current run number on file               */
  unsigned int nFrames;     /* total number of frames                   */
  unsigned long nBytes;     /* total number of bytes                    */
  struct FrIO *frfd;        /* pseudo file descriptor                   */
  int error;                /* error flag                               */
  int id;                   /* file id (it is a sequential number)      */
  char *buf;                /* pointer to the output sting. The current
                               buffer length = FrFile->buf - FrFile->p  */
  char *p;                  /* current pointer to the output string     */
  char *pStart;             /* pointer to the begining of a ord      */
  char *pMax;               /* maximum value for the output string p.   */
  long bufSize;             /* output buffer size                       */
  void (*FrSHRead)();       /* function to read FrSH structures         */
  void (*FrSERead)();       /* function to read FrSE structures         */
  struct FrStatData *sDataSim; /* static data                           */
  struct FrStatData *sDataProc;/* static data                           */
  struct FrStatData *sDataCur; /* all static data for the current frame */
  FRBOOL dicWrite[FRMAPMAXSH];/* tell if the dict. info has been written*/
  unsigned short type;      /* current type read                        */
  void *inMemF;              /* frameH frame number in the memory buffer*/
  void *inMemEF;             /* EOF frame number in the memory buffer   */
  void *inMemER;             /* EOF run number in the memory buffer     */
  void *inMemR;              /* run number in the memory buffer         */
  void *inMemS;              /* GTimeS in the memory buffer             */
  void *inMemN;              /* GTimeN in the memory buffer             */
  void *inMemH1;             /* history time (last) in the memory buffer*/
  void *inMemH2;             /* history time (previous) in the memory bu*/
  struct FrFileMark *mark;   /* file marks for Frames                   */
  FRBOOL relocation;         /* tell if we skip relocation mechanism    */
  void * address;            /* address of the last struct read         */
};

enum FrFormat {FRNATIVE,
               FRSWAP,
               FRCONTRACT,
               FREXPAND,
               FRCONTRACT_AND_SWAP,
               FREXPAND_AND_SWAP};
               
struct FrHistory {          /* Describes data history info.             */
  struct FrSH *classe;      /* class information (internal structure)   */
  char *name;               /* name of history ord                      */
  unsigned int time;        /* time of reprocessing(GPS)                */
  char *comment;            /* program name and comment                 */
  struct FrHistory *next;   /* pointer to the next history struct       */
                            /* ------------- end_of_SIO parameters -----*/
    };

struct FrMap{               /* structure used by SIO */
  void *baddress;
  int nref;
  void  *bref[FRMAPMAXREF];
};
 
struct FrObject {             /* prototype of FrameLib structure        */
  struct FrSH *classe;        /* class information (internal structure) */
  char        *name;          /* name for this element                  */
}; 

struct FrRawData {            /* Hold raw data                          */
  struct FrSH *classe;        /* class information (internal structure) */
  char   *name;               /* name                                   */
  struct FrSerData *firstSer; /* pointer to the first slow mon. station */
  struct FrAdcData *firstAdc; /*pointer to the first adc structure      */
  struct FrMsg *logMsg;       /*pointer to the error message structure  */
  struct FrVect *more;        /*pointer aditional data                  */
                              /* ------------- end_of_SIO parameters ---*/
    };
struct FrProcData {      /* onstructed data                             */
  struct FrSH *classe;   /* class information (internal structure)      */
  char   *name;          /* data name                                   */
  char   *comment;       /* comment                                     */
  double sampleRate;     /* sampling frequency (unit = Hz)              */
  unsigned int timeOffsetS;/* offset from the frame starting time (sec) */
  unsigned int timeOffsetN;/* offset from the frame starting time (nsec)*/
  double fShift;         /* frequency shift if the signal was 
                                        heterodyne (Hz) (0->no shift)   */ 
  struct FrVect *data;   /* main data like the metric perturbation (h)  */
  struct FrVect *aux;    /* auxiallary data like for h: a vector of 6 
                            values: the detector equatorial position 
                            (x,y,z) and speed (vx,vy,vz) at the first 
                            sampling.                                   */
  struct FrProcData *next; /* next bloc of onstructed data            */
                          /* ------ end_of_SIO parameters --------------*/
    };

struct FrSE {                 /* hold structure information(one per line)*/
  struct FrSH *classe;        /* class information (internal structure) */
  char        *name;          /* name for this element                  */
  char        *type;          /* element type                           */
  char        *comment;       /* comment                                */
                              /* -------- end_of_SIO parameters --------*/
  struct FrSE *next;          /* next struct E                          */
};

struct FrSH {                 /* hold info for this type of struct      */
  struct FrSH *classe;        /* class information (internal structure) */
  char   *name;               /* name for this type of structure        */
  unsigned short  id;         /* is structure when written on tape      */
  char   *comment;            /* comment                                */
                              /* -------- end_of_SIO parameters --------*/
  struct FrSE *firstE;        /* first structure element                */
  void (*objRead)();          /* function to read the object on file    */
  void (*objWrite)();         /* function to write the object from file */
  void (*objFree)();          /* function to free the object            */
  void (*objDump)();          /* function to dump the object on file    */
  void  *(*objCopy)(void *);  /* function to duplicate the object       */
  struct FrMap **map;         /* array of Frmap struct                  */
  long nMap;                  /* number of Frmap structre stored        */
  long maxMap;                /* size of map vector                     */
  void  **bAddress;
  long nAdd;                  /* number of structure                    */
  long maxAdd; 
  long nSE;                   /* number of element for this structure   */
  struct FrSH *shRef;         /* reference SH struct (used for in. file)*/
};

struct FrSimData {       /* hold Simulated data                         */
  struct FrSH *classe;   /* class information (internal structure)      */
  char   *name;          /* name                                        */
  char   *comment;       /* comment                                     */
  float   sampleRate;    /* sampling rate  (unit= Hz)                   */
  struct FrVect *data;   /*buffer for the data                          */
  struct FrVect *input;  /* input signal                                */ 
  struct FrSimData *next;/* next structure                              */
                         /* ------ end of SIO parameters ---------------*/
    }; 

struct FrSerData {            /* slow monitoring station data           */
  struct FrSH *classe;        /* class information (internal structure) */
  char *name;                 /* slow monitoring station name           */ 
  unsigned int timeSec;       /* F.B. collection time (sec since 1/1/70)*/
  unsigned int timeNsec;      /* F.B. collection time (sec since 1/1/70)*/
  float sampleRate;           /* sample/s.                              */
  char *data;                 /* pointer to the data block              */
  struct FrVect *serial;      /* additional s.m. data for this station  */
  struct FrVect *more;        /* additional s.m. data for this station  */
  struct FrSerData *next;     /* next slow monitoring station           */
                              /* ------ end of SIO parameters ----------*/
   };
struct FrStatData { /*-- this structure is used to hold quasi permanent data. 
                This data stay has long they are valid compare to the frame 
             starting time (GPS), or as long there is not a new bloc of data 
     	          with the same name more ent or with  an higgest version 
     	          number --------------------------------------------------*/
  struct FrSH *classe;      /* class information (internal structure)      */
  char   *name;             /* data type                                   */ 
  char   *comment;          /* comment 	                                   */ 
  unsigned int timeStart;   /* data validity starting time (GPS time)      */
  unsigned int timeEnd;     /* data validity end time (GPS time)           */
  unsigned int version;     /* version number                              */
  struct FrDetector *detector;/* detector owning static data               */
  struct FrVect *data;      /* data for this bloc                          */ 
                            /* ------ end_of_SIO parameters ---------------*/ 
  struct FrStatData *next;  /* next static data bloc                       */
  long nFile;
  long maxFile;
  long *fileId;
  int  overlap;
 };

struct FrVect {
  struct FrSH *classe;   /* class information (internal structure)  */
  char  *name;           /* vector name                             */
  unsigned short compress;/* 0 = no compression; 1 = gzip           */
  unsigned short type;   /* vector type  (see bellow)               */
  unsigned int  nData;   /* number of elements=nx[0].nx[1]..nx[nDim]*/
  unsigned int  nBytes;  /* number of bytes                         */
  char  *data;           /* pointer to the data area.               */
  unsigned int  nDim;    /* number of dimension                     */
  unsigned int *nx;      /* number of element for this dimension    */
  double *dx;            /* step size value (express in above unit) */
  char  **unitX;         /* unit name for (used for printout)       */
  char  *unitY;          /* unit name for (used for printout)       */
  struct FrVect *next;   /* hook for additional data                */
                         /* ------- end_of_SIO parameters ----------*/ 
  short  *dataS;         /* pointer to the data area (same as *data)*/
  int    *dataI;         /* pointer to the data area (same as *data)*/
  long   *dataL;         /* pointer to the data area (same as *data)*/
  float  *dataF;         /* pointer to the data area (same as *data)*/
  double *dataD;         /* pointer to the data area (same as *data)*/
  unsigned char  *dataU; /* pointer to the data area (same as *data)*/
  unsigned short *dataUS;/* pointer to the data area (same as *data)*/
  unsigned int   *dataUI;/* pointer to the data area (same as *data)*/
  unsigned long  *dataUL;/* pointer to the data area (same as *data)*/
  int  wSize;            /* size of one word                        */
  long  size;            /* variable for internal use               */
  void *inMemory;        /* pointer to the data in the memory buffer*/
                         /* the following variables are only fill   */
                         /*  when a vector is used in stand alonemode */
  double *startX;        /* offset for the first x value            */
  char *expName;         /* frame name (experiment name)            */
  unsigned int firstRun; /* run number for the first Frame          */
  unsigned int firstFrame;/* frame number for the first Frame       */
  unsigned int lastRun;  /* run number for the last Frame           */
  unsigned int lastFrame;/* frame number for the last Frame         */
  unsigned short ULeapS; /* leap seconds between GPS and UTC        */
  int localTime;         /* Time offset = Local time - UTC (sec)    */
};

enum FrVectTypes {FR_VECT_C,   /* vector of char           */
                  FR_VECT_2S,  /* vector of short          */
                  FR_VECT_8R,  /* vector of double         */
                  FR_VECT_4R,  /* vector of float          */
                  FR_VECT_4S,  /* vector of int            */
                  FR_VECT_8S,  /* vector of long           */
                  FR_VECT_C8,  /* vector of complex float  */
                  FR_VECT_C16, /* vector of complex double */
                  FR_VECT_S,   /* vector of string         */
                  FR_VECT_2U,  /* vector of unsigned short */
                  FR_VECT_4U,  /* vector of unsigned int   */
                  FR_VECT_8U,  /* vector of unsigned long  */
                  FR_VECT_1U,  /* vector of unsigned char  */
                  FR_VECT_END};/* vector of unsigned char  */

struct FrTrigData {     
  struct FrSH *classe;   /* class information (internal structure)      */
  char   *name;          /* TrigData name                               */
  char   *comment;       /* comment                                     */
  char   *inputs;        /* input channels                              */
  unsigned int GTimeS;   /* Event occurrence time (second) (GPS time)   */
  unsigned int GTimeN;   /* Event occurrence time (nano second)         */
  unsigned int bvalue;   /* value returned by the trigger               */
  float  rvalue;         /* value returned by the trigger               */
  float  probability;    /* lieklihood estimate of event                */
  char   *statistics;    /* stat. description of event                  */
  struct FrVect *data;   /* vector containing additional trigger results */
  struct FrTrigData *next;/* next TrigData structure                      */
                         /* -------- end_of_SIO parameters -------------*/ 
    }; 

struct FrSummary {     
  struct FrSH *classe;   /* class information (internal structure)      */
  char   *name;          /* summary name                                */
  char   *comment;       /* comment                                     */
  char   *test;          /* statistical test(s) used on raw data        */
  struct FrVect *moments;/* vector containing statistical description   */
  struct FrSummary *next;/* next summary structure                      */
                         /* -------- end_of_SIO parameters -------------*/ 
    }; 


/*----------------------------------------------------------------------*/
FILE *FrLibIni(char *outFile, FILE *fOut, long dbglvl);
void FrLibShortIni();
void FrLibSetLvl(long dbglvl);
float FrLibVersion(FILE *fOut);
/*----------------------------------------------------------------------*/
void   FrFileDbg(struct FrFile *file);
void   FrFileFree(struct FrFile *file);
struct FrFile *FrFileNew(char *fileName, unsigned short compress, 
                         char *buf, long bufSize);
void   FrFileNextName(struct FrFile *file);
long   FrFileId(long id, long mode);

void   FrFileIEnd(struct FrFile *iFile);
struct FrFile *FrFileINew(char *fileName, ...);
struct FrFile *FrFileINewFd(struct FrIO *frfd, ...);
void   FrFileIncSH(struct FrFile *iFile, unsigned short id);
void   FrFileIOpen(struct FrFile *file);
struct FrFile *FrFileIRewind(struct FrFile *iFile);

void   FrFileOEnd(struct FrFile *file);
struct FrFile *FrFileONew(char *fileName, unsigned short compress, 
                                          char *buf, long bufSize);
struct FrFile *FrFileONewFd(struct FrIO *frfd, unsigned short compress, 
                                          char *buf, long bufSize);
void FrFileOOpen(struct FrFile *oFile);
void FrFileOUpdate(struct FrFile *oFile,unsigned int frame, 
              unsigned int run, unsigned int gtimeS, unsigned int gtimeN);
void FrFileOSetGzipLevel(struct FrFile *oFile, unsigned short level);
/*----------------------------------------------------------------------*/
struct FrAdcData *FrAdcDataCopy(struct FrAdcData *in, struct FrameH *frame);
struct FrSH *FrAdcDataDef();
struct FrAdcData *FrAdcDataFind(struct FrameH *frameH, char *adcName); 
struct FrAdcData *FrAdcDataFree(struct FrAdcData *adcData);
struct FrAdcData *FrAdcDataNew(struct FrameH *frame, 
               char *name, double sampleRate, long nData, long nBits);
void FrAdcDataRead(struct FrFile *iFile);
void FrAdcDataWrite(struct FrAdcData *adcData, struct FrFile *oFile);
/*----------------------------------------------------------------------*/
unsigned short FrVectCompData(struct FrVect *vect,int compType,
         int gzipLevel, unsigned char **result, unsigned int *nBytes);
void FrameCompress(struct FrameH *frameH, int compress, int gzipLevel);
struct FrameH *FrameCopy(struct FrameH *frameH);
void FrameDump(struct FrameH *frameH, FILE *fp, long debugLvl);
char *FrameDumpToBuf(struct FrameH *frame, long debugLvl, char *buf, 
            long bufSize);  
void FrameExpand(struct FrameH *frameH);
void FrameFree(struct FrameH *FrameH);
struct FrameH *FrameRead(struct FrFile *iFile);
struct FrameH *FrameReadFromBuf(char *buf, long nBytes, unsigned short comp);
void FrameSelectAdc(struct FrameH *frameH,int nAdc, char *adc1, ...);
int FrameWrite(struct FrameH *frameH, struct FrFile *oFile);
long FrameWriteToBuf(struct FrameH *frameH, unsigned short comp,
                    char *buf, long nBytes);

struct FrameH *FrameHCopy(struct FrameH *in);
struct FrSH *FrameHDef();    
void FrameHRead(struct FrFile *iFile);
struct FrameH *FrameHNew(char *name);
void FrameHWrite(struct FrameH *frameH, struct FrFile *oFile);
/*----------------------------------------------------------------------*/
struct FrSH *FrDetectorDef();    
void FrDetectorDump(struct FrDetector *detector, FILE *fp, long debugLvl);
struct FrDetector *FrDetectorNew(char *name);
void FrDetectorFree(struct FrDetector *detector);
void FrDetectorRead(struct FrFile *iFile);
void FrDetectorWrite(struct FrDetector *detector, struct FrFile *oFile);
/*----------------------------------------------------------------------*/
struct FrHistory *FrHistoryAdd(struct FrameH *frame, char *comment);
struct FrSH *FrHistoryDef();    
struct FrHistory *FrHistoryFree(struct FrHistory *history);
void   FrHistoryRead(struct FrFile *iFile);
void   FrHistoryWrite(struct FrHistory *history, struct FrFile *oFile);
/*----------------------------------------------------------------------*/
struct FrMsg *FrMsgAdd(struct FrameH *frame, char *alarm, char *message, 
                       unsigned int severity);
struct FrSH *FrMsgDef();    
void FrMsgFree(struct FrMsg *msg);
void FrMsgRead(struct FrFile *iFile);
void FrMsgWrite(struct FrMsg *msg, struct FrFile *oFile);
/*----------------------------------------------------------------------*/
struct FrSH *FrObjectDef();
void FrObjectFree (struct FrObject *proto);
void FrObjectDump (struct FrObject *proto, FILE *fp);
void FrObjectWrite(struct FrObject *proto, struct FrFile *oFile);
/*----------------------------------------------------------------------*/
struct FrMap *FrSetAddMap(struct FrSH *sh, unsigned short id);
void FrSetAll(struct FrFile *file);
void FrSetBAT(struct FrFile *file, unsigned short instance, void *add);
void FrSetBRT(struct FrFile *file, unsigned short instance, 
                                   unsigned short type, void *add);
void FrSetIni(struct FrFile *file);
/*----------------------------------------------------------------------*/
struct FrSH *FrEndOfFileDef();
void FrEndOfFileRead(struct FrFile *iFile);
void FrEndOfFileWrite(struct FrFile *oFile);
/*----------------------------------------------------------------------*/
struct FrSH *FrEndOfFrameDef();    
void FrEndOfFrameRead(struct FrFile *iFile);
void FrEndOfFrameWrite(struct FrameH *frameH, struct FrFile *oFile);
/*----------------------------------------------------------------------*/
char *FrError(long level, char *subroutine, char *message);
char *FrErrorGetHistory();
void  FrErrorDefHandler(long level, char *lastMessage);
void  FrErrorSetHandler(void  (*handler)(long, char *));
/*----------------------------------------------------------------------*/
struct FrSH *FrDicAddS(char *type, void (*funRead)());         
long FrDicGetId(struct FrSH *type, void *instance);
void FrDicIni();
/*----------------------------------------------------------------------*/
struct FrSH *FrRawDataDef();    
void FrRawDataDump(struct FrRawData *rawData, FILE *fp, long debugLvl);
void FrRawDataFree(struct FrRawData *rawData);
struct FrRawData *FrRawDataNew(struct FrameH *frame);
void FrRawDataRead(struct FrFile *iFile);
void FrRawDataWrite(struct FrRawData *rawData, struct FrFile *oFile);
/*----------------------------------------------------------------------*/
struct FrSH *FrProcDataDef();
struct FrProcData *FrProcDataFind(struct FrameH *frameH, char *name);
void FrProcDataFree(struct FrProcData *procData);
struct FrProcData *FrProcDataNew(struct FrameH *frame, char *name, 
                   double sampleRate, long nData, long nBits);
void FrProcDataRead(struct FrFile *iFile);
void FrProcDataWrite(struct FrProcData *procData, struct FrFile *oFile);
/*----------------------------------------------------------------------*/
struct FrSH *FrSEDef();
void FrSENew(struct FrSH *structH, char *type, char *name, char *comment);
void FrSERead(struct FrFile *iFile);
void FrSEWrite(struct FrSE *structE, struct FrFile *oFile);
/*----------------------------------------------------------------------*/
struct FrSH *FrSHDef();
void FrSHMatch(struct FrFile *iFile, unsigned short type);
struct FrSH *FrSHNew(char *name, char *comment);
void FrSHRead(struct FrFile *iFile);
void FrSHWrite(struct FrSH *structH, struct FrFile *oFile);
/*----------------------------------------------------------------------*/
struct FrSH *FrSimDataDef();    
struct FrSimData *FrSimDataFind(struct FrameH *frameH, char *simName);
void FrSimDataFree(struct FrSimData *simData);
struct FrSimData *FrSimDataNew(struct FrameH *frame, char *name, 
                   double sampleRate, long nData, long nBits);
void FrSimDataRead(struct FrFile *iFile);
void FrSimDataWrite(struct FrSimData *adcData, struct FrFile *oFile);
/*----------------------------------------------------------------------*/
struct FrSH *FrSerDataDef();  
struct FrSerData *FrSerDataFind(struct FrameH *frameH,char *smsName);
void FrSerDataFree(struct FrSerData *smsData);
long FrSerDataGet(struct FrameH *FrameH, char *smsName, 
                  char *smsParam, double *value); 
struct FrSerData *FrSerDataNew(struct FrameH *frame,
                              char *smsName, unsigned int serTime, char *data);
void FrSerDataUpd(struct FrSerData *smsData, unsigned int serTime, char *data);
void FrSerDataRead(struct FrFile *iFile);
void FrSerDataWrite(struct FrSerData *smsData, struct FrFile *oFile);
/*------------------------------------------------------------------*/
void FrStatDataAdd(struct FrDetector *detector, struct FrStatData *sData);
void FrStatDataChkT(struct FrStatData **root, 
                    unsigned int timeStart, unsigned int timeEnd);
struct FrStatData *FrStatDataCopy(struct FrStatData *sData);
struct FrSH *FrStatDataDef();    
struct FrStatData *FrStatDataFind(struct FrDetector *detector, char *name,
                     unsigned int timeNow);
struct FrStatData *FrStatDataFree(struct FrStatData *sData);
struct FrStatData *FrStatDataNew(char *name, char *comment, unsigned int  tStart,
                         unsigned int  tEnd, unsigned int  version, void *data);
void FrStatDataRead(struct FrFile *iFile);
void FrStatDataRemove(struct FrDetector *detector, char *name);
void FrStatDataTouch(struct FrStatData *sData);
void FrStatDataWrite(struct FrStatData *sData, struct FrFile *oFile);
/*------------------------------------------------------------------*/
char *FrStrCpy(char **copy, char *old);
char *FrStrGTime(unsigned int in);
/*------------------------------------------------------------------*/
struct FrSH *FrSummaryDef();
struct FrSummary *FrSummaryFind(struct FrameH *frameH, char *name);
struct FrSummary *FrSummaryFree(struct FrSummary *summary);
struct FrSummary *FrSummaryNew(struct FrameH *frameH,  
     char *name,  char *comment,  char *test, struct FrVect *moments);
void FrSummaryRead(struct FrFile *iFile);
void FrSummaryWrite(struct FrSummary *summary, struct FrFile *oFile);
/*------------------------------------------------------------------*/
struct FrSH *FrTrigDataDef();
struct FrTrigData *FrTrigDataFind(struct FrameH *frameH, char *name);
struct FrTrigData *FrTrigDataFree(struct FrTrigData *trigData);
struct FrTrigData *FrTrigDataNew(struct FrameH *frameH,  
  char *name, char *comment, char *inputs, 
  unsigned int GTimeS, unsigned int GTimeN, unsigned int bvalue,
  float  rvalue, float  probability, char   *stat, 
  struct FrVect *data);
void FrTrigDataRead(struct FrFile *iFile);
void FrTrigDataWrite(struct FrTrigData *trigData, struct FrFile *oFile);
/*------------------------------------------------------------------*/
void FrVectCompress(struct FrVect *vect,int compress, int gzipLevel);
struct FrVect *FrVectCopy(struct FrVect *in);
struct FrSH *FrVectDef();    
char *FrVectDiff(struct FrVect *vect);
void FrVectDump(struct FrVect *vect, FILE *fp, long debugLvl);
void FrVectExpand(struct FrVect *vect);
void FrVectFillC(struct FrVect *vect,  char value);
void FrVectFillI(struct FrVect *vect,   int value);
void FrVectFillS(struct FrVect *vect, short value);
void FrVectFree(struct FrVect *vect);
void FrVectInt(struct FrVect *vect);
void FrVectMap(struct FrVect *vect);
struct FrVect *FrVectNew(long type,long nDim, ...);
struct FrVect *FrVectNewTS(char *name,double sampleRate,
                              long nData,long nBits);
struct FrVect *FrVectNew1D(char *name,long type, long nData, 
                           double dx, char *unitx, char *unity);
void FrVectRead(struct FrFile *iFile);
void FrVectWrite(struct FrVect *v, struct FrFile *oFile);
int FrVectZComp(unsigned short *out, unsigned long *compL,
                short *data, int nData, int bSize);
void FrVectZExpand(short *out, unsigned short *data, int nData);

/*------ here we have the I/O package. ------------ */
   
void FrPutNewRecord(struct FrFile *oFile, void *instance);
void FrPutChar  (struct FrFile *oFile,   char value); 
void FrPutDouble(struct FrFile *oFile, double value);
void FrPutFloat (struct FrFile *oFile, float value);
void FrPutInt   (struct FrFile *oFile,    int value);
void FrPutIntU  (struct FrFile *oFile, unsigned int value);
void FrPutLong  (struct FrFile *oFile,   long value);
void FrPutShort (struct FrFile *oFile,  short value);
void FrPutShortU(struct FrFile *oFile, unsigned short value);
void FrPutSChar (struct FrFile *oFile,  char *value);
void FrPutStruct(struct FrFile *oFile, void *instance);

void FrPutVC( struct FrFile *oFile, char      *data, long nData);
void FrPutVD( struct FrFile *oFile, double    *data, long nData);
void FrPutVF( struct FrFile *oFile, float     *data, long nData);
void FrPutVI( struct FrFile *oFile, int       *data, long nData);
void FrPutVL( struct FrFile *oFile, long      *data, long nData);
void FrPutVQ( struct FrFile *oFile, char     **data, long nData);
void FrPutVS( struct FrFile *oFile, short     *data, long nData);

void FrPutWriteRecord( struct FrFile *oFile);

/*------------------------------------------------------------------*/

void FrReadHeader(struct FrFile *iFile, void *instance);

int  FrRead(struct FrFile *iFile, char *buf, unsigned int n);
void FrReadDouble(struct FrFile *iFile,       double *value);
void FrReadFloat( struct FrFile *iFile,        float *value);
void FrReadInt(   struct FrFile *iFile,          int *value);
void FrReadIntU(  struct FrFile *iFile, unsigned int *value);
void FrReadLong(  struct FrFile *iFile,         long *value);
void FrReadRecord(struct FrFile *iFile, unsigned short type, unsigned int length);
void FrReadSChar( struct FrFile *iFile,  char **value);
void FrReadShort( struct FrFile *iFile,  short *value);
void FrReadShortU(struct FrFile *iFile,  unsigned short *value);
void FrReadStruct(struct FrFile *iFile, void *add);
void FrReadVC(    struct FrFile *iFile, char      **data, unsigned int  nData);
void FrReadVD(    struct FrFile *iFile, double    **data, unsigned int  nData);
void FrReadVF(    struct FrFile *iFile, float     **data, unsigned int  nData);
void FrReadVI(    struct FrFile *iFile, int       **data, unsigned int  nData);
void FrReadVL(    struct FrFile *iFile, long      **data, unsigned int  nData);
void FrReadVQ(    struct FrFile *iFile, char     ***data, unsigned int  nData);
void FrReadVS(    struct FrFile *iFile, short     **data, unsigned int  nData);

/*--------------------------------------------------------------------*/
void FrBack2ResetRead(struct FrFile *file);
void FrBack2FileOpen(struct FrFile *file);
void FrBack2FileHeader(unsigned char *buf);
void FrBack2SHRead(struct FrFile *iFile);
void FrBack2FrameHRead(struct FrFile *iFile);
void FrBack2AdcDataRead(struct FrFile *iFile);
void FrBack2DetectorRead(struct FrFile *iFile);
void FrBack2EndOfFrameRead(struct FrFile *iFile);
void FrBack2HistoryRead(struct FrFile *iFile);
void FrBack2MsgRead(struct FrFile *iFile);
void FrBack2ProcDataRead(struct FrFile *iFile);
void FrBack2SerDataRead(struct FrFile *iFile);
void FrBack2SimDataRead(struct FrFile *iFile);
void FrBack2StatDataRead(struct FrFile *iFile);
void FrBack2VectRead(struct FrFile *iFile);
void FrBack3FrameHRead(struct FrFile *iFile);
void FrBack3FrStatDataRead(struct FrFile *iFile);

#ifdef __cplusplus
}
#endif

#endif
















