/*-----------------------------------------------------------*/
/* File: exampleFull.c          by B. Mours(LAPP) Sep 10, 99 */
/*                                                           */
/*  This program creates a frame and write it on file        */
/*-----------------------------------------------------------*/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "FrameL.h"
  
int main()     /*-------------------- main-----------*/
{struct FrFile *oFile;
 struct FrameH *frame;
 struct FrAdcData *adc, *adc5;
 time_t utc;
 double sampleRate,phase;
 long nData,buffSize,i,j,nADC,sec,nData5;
 char *buff,name[100];
 
    /*------ initialisation -------------------------------*/
 
  if(FrLibIni("full.lis",NULL, 1) == NULL)
     {printf("Error during initialisation\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
      return(0);}

    /*----- create one Frame header and set its time-------*/
    
  frame = FrameHNew("demo");
  if(frame == NULL)
     {printf("Error during frame creation\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
      return(0);}

  frame->ULeapS    = FRGPSLEAPS;
  utc = time(NULL);
  frame->localTime = mktime(localtime(&utc)) - mktime(gmtime(&utc));
  frame->GTimeS    = utc - FRGPSOFF + frame->ULeapS;

   /*----- add 10 fast (20kHz) ADC with data provided 
                          y a random generator ------------*/

  sampleRate = 2.e+4;
  nData = 65536;
  frame->dt = nData/sampleRate;  
  nADC = 10;

  for(i=0; i<nADC; i++)
     {sprintf(name,"fastAdc%ld",i);
      adc = FrAdcDataNew(frame,name,sampleRate,nData,16);
      if(adc == NULL) break;
      for(j=0; j < nData; j++)
         {adc->data->dataS[j] = (short) (rand());}}

    /*---- reserve space for slower (1250 Hz) ADC's ------*/

  nData = nData / 16;
  sampleRate = sampleRate/16.;
  nADC = 10;
  for(i=0; i<nADC; i++)
     {sprintf(name,"Adc%ld",i);
      adc = FrAdcDataNew(frame,name,sampleRate,nData,16);
      if(i == 5) {adc5 = adc;}}
  nData5 = nData;

    /*---- reserve space for very slow (1 Hz) ADC's -----*/

  nData = nData / 1024;
  sampleRate = sampleRate/1024.;
  nADC = 10;
  for(i=0; i<nADC; i++)
     {sprintf(name,"slowAdc%ld",i);
      FrAdcDataNew(frame,name,sampleRate,nData,12);}

    /*---- reserve space for various type of data ------*/

  FrAdcDataNew(frame,"D1",sampleRate,nData,-32);
  FrSimDataNew(frame,"sim1",sampleRate,nData,32);
  FrSerDataNew(frame,"sms1",0,"sms data are here");
  FrMsgAdd(frame,"test","no message",0);
  FrSummaryNew(frame,"Quality 1","main quality"," ",NULL);
  FrSummaryNew(frame,"Quality 2","aux"," ",NULL);
  FrTrigDataNew(frame,"trigger 1","Binary search","Main strain",
           0,0,0,1.,1.e-20,"match filtering",NULL);
  FrTrigDataNew(frame,"trigger 2","Binary search","Main strain",
           0,0,0,1.,2.e-20,"match filtering",NULL);
    /*----- dump on stdout the current frame  ----------*/

  FrameDump(frame, stdout,3);

    /*----- open the output file in binary format 
            with differentiation and compression ------*/

  buffSize = 500000;
  buff = malloc(buffSize);
                              
  oFile = FrFileONew("ran.dat", 0, buff, buffSize); 
  if(oFile == NULL)
     {printf("Error during output file creation\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
      return(0);}

    /*---------------------- output 10 frames  -----------------*/
    /*(they are all identical except the frame number and Adc5) */
    
  for(i=0; i<10; i++)
     {     
      for(j=0; j < nData5; j++)
         {phase = .023*(double)(j + i*nData5);
          adc5->data->dataS[j] = 273.*sin(phase) + (short)(rand())/1000;}

      printf(" output frame %d\n",frame->frame);
      if(FrameWrite(frame, oFile) != FR_OK)
          {printf("Error during frame write\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
           return(0);}
      frame->frame++;
      frame->GTimeS += (unsigned int) (frame->dt);
      frame->GTimeN += ((unsigned int) (1.e+9*frame->dt)) % 1000000000;
      sec = frame->GTimeN/1000000000;
      frame->GTimeS += sec;
      frame->GTimeN -= sec*1000000000;}

    /*----- close everything -------------------------*/
     
  FrFileOEnd(oFile);
 
  FrameFree(frame);

  return(0);
}
