/*-----------------------------------------------------------*/
/* File: exampleOnlineUpd.c      by B.Mours (LAPP) Sep 10,99 */
/*                                                           */
/*  This program creates and write in memory several frames  */
/*  The frames are directly updated in memory. Warning: this */
/*  should be used only by expert.                           */
/*-----------------------------------------------------------*/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h> 
#include "FrameL.h"

#define NDATA 1024
  
int main()     /*-------------------- main------------------*/
{struct FrFile *oFile;
 struct FrameH *frame;
 struct FrAdcData *adc1,*adc2;
 double frequency,phase1,phase2;
 long nData,buffSize,j;
 char *buff,fileName[100];
 short dataS[NDATA];
 unsigned int gtimeS,gtimeN,fNum,sec;
 int fd,frameSize;
 
    /*----- create one Frame header ------------------------*/
    
  frame = FrameHNew("new");
  frame->frame = 15;
  frame->run = 8;

  FrHistoryAdd(frame,"Online Frame Builder"); 

   /*-----reserve space for 2 ADC ---------------*/

  frequency = 1.e3;
  nData = NDATA; 
  frame->dt = nData/frequency;

  adc1 = FrAdcDataNew(frame,"adc1",frequency,nData,16);
  adc2 = FrAdcDataNew(frame,"adc2",frequency,nData,16);
  if(adc2 == NULL)
         {printf("Error during adc creation\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
          return(0);}
  
    /*--reserve space for the output internal buffer ------*/

  buffSize = 4*nData + 500000;
  buff = malloc(buffSize);

    /*---- write the 'template' frame in the internal buffer----*/
    
  oFile = FrFileONew(NULL , 0, buff, buffSize); 
  if(oFile == NULL)
        {printf("Error during output file creation\n"
               "  Last errors are:\n%s",FrErrorGetHistory());
         return(0);}

  if(FrameWrite(frame, oFile) != FR_OK)
          {printf("Error during frame write\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
           return(0);}
 
  FrFileOEnd(oFile);

  frameSize = oFile->p - buff;
  printf(" The frame size is %d bytes\n", frameSize);

  /*---- now we start the loop on 10 frames ------------*/

  gtimeS = frame->GTimeS;
  gtimeN = 0.; 
  phase1 = 0.;
  phase2 = 0.;

  for(fNum=0; fNum<10; fNum++)

    /*--- first update the frameH and the history info --*/

     {gtimeS += (unsigned int) (frame->dt);
      gtimeN += ((unsigned int) (1.e+9*frame->dt)) % 1000000000;
      sec = gtimeN/1000000000;
      gtimeS += sec;
      gtimeN -= sec*1000000000;

     FrFileOUpdate(oFile, fNum, frame->run, gtimeS, gtimeN);

      /*---- put in adc1 and adc2 sin waves ------------*/

      for(j=0; j < nData; j++)
         {phase1 += .0123;
          dataS[j] = (short)(1000.*cos(phase1));}
      memcpy(adc1->inMemory,dataS,nData*2);

      for(j=0; j < nData; j++)
         {phase2 += .00723;
          dataS[j] = (short)(1000.*cos(phase2));}
      memcpy(adc2->inMemory,dataS,nData*2);

      /*--- write frame on disk   -------------------*/

      sprintf(fileName,"frame%d.dat",fNum);
      fd = open(fileName, O_CREAT|O_WRONLY,0644);
      write(fd, buff, frameSize);
      close(fd);}     

 return(0);}
