/*-----------------------------------------------------------*/
/* File: exampleStat.c      by B. Mours(LAPP) Sep 10, 99     */
/*                                                           */
/*  This program produces frames with static data.           */
/*-----------------------------------------------------------*/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "FrameL.h"
  
int main()     /*-------------------- main-------------------*/
{struct FrFile *oFile;
 struct FrameH *frame;
 struct FrAdcData *adc1;
 struct FrVect *gData;
 struct FrStatData *gain;
 time_t utc;
 double sampleRate, phase;
 long nData,buffSize,i,j,k;
 unsigned int t0,dtS,dtN,sec;
 char *buff;
 
    /*------ initialisation --------------------------------*/
 
  if(FrLibIni(NULL, stdout, 1) == NULL)
     {printf("Error during initialisation\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
      return(0);}

    /*----- create one Frame header ------------------------*/
    
  frame = FrameHNew("Test data");
  if(frame == NULL)
     {printf("Error during frame creation\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
      return(0);}

  frame->ULeapS    = FRGPSLEAPS;
  utc = time(NULL);
  frame->localTime = mktime(localtime(&utc)) - mktime(gmtime(&utc));
  frame->GTimeS    = utc - FRGPSOFF + frame->ULeapS;

  FrHistoryAdd(frame,"created by a test program");

   /*------add detector information -----------------------*/

  frame->detectProc = FrDetectorNew("Test Interferometer");
  frame->detectProc->armLength = 6.;

   /*----- add 1 fast ADC with data provided by a 
           random generator or a sine wave --*/

  sampleRate = 2.e+4;
  nData = 65536;
  frame->dt = nData/sampleRate;  
  dtS = (unsigned int) (frame->dt);
  dtN = ((unsigned int) (1.e+9*frame->dt)) % 1000000000;

  adc1 = FrAdcDataNew(frame,"adc 1",sampleRate,nData,16);
  phase = 0.1532;

   /*------- add static data -----------------------------*/
   /*--(in this case the ADC gain for each channel)-------*/

  t0 = frame->GTimeS;
  gData = FrVectNew(FR_VECT_4R,1,nData,1.,"relatif_gain");
  gain  = FrStatDataNew("gain","ADC 1 gain",t0,t0+20,0,gData);
  FrStatDataAdd(frame->detectProc, gain);

  for(i=0; i<5; i++) 
       {gData->dataF[i] = 1.1;}


    /*----- open the output file in binary format --------*/

  buffSize = 500000;
  buff = malloc(buffSize);
                              
  oFile = FrFileONew("ran.dat", 0, buff, buffSize); 

    /*-------generate and output 10 frames  --------------*/
    
  for(i=0; i<10; i++)
     {
       /*---- we first update the frame data -------------*/

      frame->frame++;
      frame->GTimeS += dtS;
      frame->GTimeN += dtN;
      sec = frame->GTimeN/1000000000;
      frame->GTimeS += sec;
      frame->GTimeN -= sec*1000000000;

      for(j=0; j < nData; j++)
        {phase = phase + 1.934e-3;
         adc1->data->dataS[j] = (short)(rand()+2000.*sin(phase));}

        /*---- we simulate de generation of new calibration data:
               we change the calibration constant every 10 seconds.  --*/

      if(frame->GTimeS > gain->timeEnd)
         {gain->timeStart = gain->timeEnd;
          gain->timeEnd   = gain->timeStart + 20;
          FrStatDataTouch(gain);
          for(k=0; k<nData; k++)
             {gData->dataF[k] += .0002;}}

    /*----- dump on stdout the current frame  -----------*/

     FrameDump(frame, stdout,2);

        /*--- then we write the data on file ------------*/

      printf(" output frame %d\n",frame->frame);

      if(FrameWrite(frame, oFile) != FR_OK)
          {printf("Error during frame write\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
           return(0);}
      }

    /*----- close everything ----------------------------*/
     
  FrFileOEnd(oFile);
                                
  FrameFree(frame);

  return(0);
}
